﻿using InstanceFactory.CustomPrivateAccessor.ConsoleApplication._4._5;
using InstanceFactory.CustomPrivateAccessor.ConsoleApplication.Test._4._5.TestTools.UnitTesting;
using System.Threading.Tasks;

namespace InstanceFactory.CustomPrivateAccessor.ConsoleApplication.Test._4._5.TestTools.Accessors
{
  /// <summary>
  /// Accessor for ClassWithNonPublicMembers.
  /// </summary>
  internal class ClassWithNonPublicMembersAccessor : PrivateAccessorBase<ClassWithNonPublicMembers>
  {
    #region Interal Constructors

    /// <summary>
    /// Initializes a new instance of <see cref="ClassWithNonPublicMembersAccessor"/> with an instance of the class to be tested.
    /// </summary>
    internal ClassWithNonPublicMembersAccessor()
      : base(new ClassWithNonPublicMembers())
    {
    }

    #endregion Interal Constructors


    #region Internal Properties

    /// <summary>
    /// Gives get / set access to the property "ProtecedProperty".
    /// </summary>
    internal string ProtectedProperty
    {
      get { return (GetPropertyValue<string>()); }
      set { SetPropertyValue(value); }
    }

    /// <summary>
    /// Gives get / set access to the property "PrivateProperty".
    /// </summary>
    internal string PrivateProperty
    {
      get { return (GetPropertyValue<string>()); }
      set { SetPropertyValue(value); }
    }

    #endregion Internal Properties


    #region Internal Static Properties

    /// <summary>
    /// Gives get / set access to the static property "PrivateStaticProperty".
    /// </summary>
    internal static string PrivateStaticProperty
    {
      get { return (PrivateAccessorBase<ClassWithNonPublicMembers>.GetStaticPropertyValue<string>()); }
      set { PrivateAccessorBase<ClassWithNonPublicMembers>.SetStaticPropertyValue(value); }
    }

    #endregion Internal Static Properties


    #region Internal Methods

    /// <summary>
    ///  Calls the protected method ProtectedMethod
    /// </summary>
    /// <param name="parameter">The parameter</param>
    /// <returns>The return value of the called method.</returns>
    internal string ProtectedMethod
      (
      string parameter
      )
    {
      // Use the base class to invoke the correct method.
      // Need to name the parameter, otherwise the first parameter will be interpreted as caller's name.
      return (Invoke<string>(parameter: parameter));
    }

    /// <summary>
    ///  Calls the private method PrivateMethodWithReturnValueAsync
    /// </summary>
    /// <param name="millisecondsDelay">Delaytime in milliseconds..</param>
    /// <param name="outputText">Output text.</param>
    /// <returns>Delaytime in seconds.</returns>
    internal async Task<double> PrivateMethodWithReturnValueAsync
      (
      int millisecondsDelay,
      string outputText
      )
    {
      // Invoke the method
      double returnValue = await InvokeAsync<double>(parameter: new object[] { millisecondsDelay, outputText });

      // Return the result.
      return (returnValue);
    }

    #endregion Internal Methods


    #region Internal Static Methods

    /// <summary>
    ///  Calls the private method PrivateMethodWithReturnValueAsync
    /// </summary>
    /// <param name="millisecondsDelay">Delaytime in milliseconds..</param>
    /// <param name="outputText">Output text.</param>
    /// <returns>Delaytime in seconds.</returns>
    internal static async Task PrivateStaticVoidMethodAsync
      (
      int millisecondsDelay,
      string outputText
      )
    {
      // Invoke the method
      await PrivateAccessorBase<ClassWithNonPublicMembers>.InvokeStaticAsync(parameter: new object[] { millisecondsDelay, outputText });
    }

    #endregion Internal Static Methods
  }
}
