﻿using InstanceFactory.CustomPrivateAccessor.ConsoleApplication.Test.TestTools.UnitTesting;
using System.Runtime.CompilerServices;

namespace InstanceFactory.CustomPrivateAccessor.ConsoleApplication.Test.TestTools.Accessors
{
  /// <summary>
  /// Accessor for ClassWithNonPublicMembers.
  /// </summary>
  internal class ClassWithNonPublicMembersAccessor : PrivateAccessorBase<ClassWithNonPublicMembers>
  {
    #region Interal Constructors

    /// <summary>
    /// Initializes a new instance of <see cref="ClassWithNonPublicMembersAccessor"/> with an instance of the class to be tested.
    /// </summary>
    internal ClassWithNonPublicMembersAccessor()
      : base(new ClassWithNonPublicMembers())
    {
    }

    #endregion Interal Constructors


    #region Internal Properties

    /// <summary>
    /// Gives get / set access to the property "ProtecedProperty".
    /// </summary>
    internal string ProtectedProperty
    {
      [MethodImpl(MethodImplOptions.NoInlining)]
      get { return (GetPropertyValue<string>()); }
      [MethodImpl(MethodImplOptions.NoInlining)]
      set { SetPropertyValue(value); }
    }

    /// <summary>
    /// Gives get / set access to the property "PrivateProperty".
    /// </summary>
    internal string PrivateProperty
    {
      [MethodImpl(MethodImplOptions.NoInlining)]
      get { return (GetPropertyValue<string>()); }
      [MethodImpl(MethodImplOptions.NoInlining)]
      set { SetPropertyValue(value); }
    }

    #endregion Internal Properties


    #region Internal Static Properties

    /// <summary>
    /// Gives get / set access to the static property "PrivateStaticProperty".
    /// </summary>
    internal static string PrivateStaticProperty
    {
      [MethodImpl(MethodImplOptions.NoInlining)]
      get { return (PrivateAccessorBase<ClassWithNonPublicMembers>.GetStaticPropertyValue<string>()); }
      [MethodImpl(MethodImplOptions.NoInlining)]
      set { PrivateAccessorBase<ClassWithNonPublicMembers>.SetStaticPropertyValue(value); }
    }

    #endregion Internal Static Properties


    #region Internal Methods

    /// <summary>
    ///  Calls the protected method ProtectedMethod
    /// </summary>
    /// <param name="parameter">The parameter</param>
    /// <returns>The return value of the called method.</returns>
    [MethodImpl(MethodImplOptions.NoInlining)]
    internal string ProtectedMethod
      (
      string parameter
      )
    {
      // Use the base class to invoke the correct method.
      return (Invoke<string>(parameter));
    }

    #endregion Internal Methods
  }
}
